// mqtt_neopixel.ino --- 16.09.2018 ------------------------------------
//

#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include <Adafruit_NeoPixel.h>

// hier stehen ssid, passwd, netadr, topic, clientname
// #include "secrets.h"

// -- alternativ Definition an dieser Stelle:
#define WIFI_SSID  "ssid"
#define WIFI_PASSWD "passwd"
#define WIFI_ADR "192.168.x.x"
#define MQTT_TOPIC  "x/y"
#define MQTT_CLIENT_NAME "neopixel"

//#define DEBUG               // Debugausgabe aktivieren
// -------------------------------------------------------------
#define NEOPIXEL_STATIC      // die Neopixels ohne Blinken aktivieren
#define NEOPIXEL_BLINK       // die Blinkoption der Neopixels zuschalten
#define NEOPIXEL_PIN    2    // ESP-01
#define NEOPIXELS      16    // Anzahl der Neopixel-LED's

uint8_t pixel[] = {0,0,0,0,0,0}; 

Adafruit_NeoPixel pixels = Adafruit_NeoPixel(
                  NEOPIXELS, NEOPIXEL_PIN, NEO_GRB + NEO_KHZ800);

// --------------------------------------------------------------
// Daten des WLANs und MQTT-Servers (ausgelagert in secrets.h)
const char* ssid        = WIFI_SSID;
const char* password    = WIFI_PASSWD;

const char* mqtt_server = WIFI_ADR;
const char* topic       = MQTT_TOPIC;
const char* mqtt_client = MQTT_CLIENT_NAME;

WiFiClient espClient;
PubSubClient client(espClient);

// --------------------------------------------------------
void wifi_connect(void)
{
#ifdef DEBUG
  Serial.println("Connecting WiFi ...");
#endif
  while (WiFi.status() != WL_CONNECTED) 
  {
    // während Suche nach Router ROT blinken
    pixels.setPixelColor(0, pixels.Color(50, 0, 0));
    pixels.show();
    delay(200);
    pixels.setPixelColor(0, pixels.Color(0, 0, 0));
    pixels.show();
    delay(200);

  #ifdef DEBUG
    Serial.print("+");
  #endif
  }
#ifdef DEBUG
  Serial.println();
  Serial.println("WiFi connected !");
#endif
}


// ------------------------------------------------------------
void mqtt_connect() 
{
#ifdef DEBUG
  Serial.println("Connecting MQTT ...");
#endif
  while (!client.connected()) 
  {
    if (client.connect(mqtt_client)) 
    {
      client.subscribe(topic);
      // Beim Connect kurz GRÜN aufleuchten
      pixels.setPixelColor(0, pixels.Color(0, 50, 0));
      pixels.show();
      delay(200);
      } 
      else 
      {
      // Fehlende Verbindung BLAU blinkend anzeigen
      pixels.setPixelColor(0, pixels.Color(0, 0, 50));
      pixels.show();
      delay(200);
      pixels.setPixelColor(0, pixels.Color(0, 0, 0));
      pixels.show();
      delay(200);
    #ifdef DEBUG
      Serial.print(".");
    #endif
    }
  }
  pixels.setPixelColor(0, pixels.Color(0, 0, 0));
  pixels.show();
#ifdef DEBUG
  Serial.println();
  Serial.println("MQTT connected !");
#endif
}


// ---------------------------------------------------------
void setup()
{
#ifdef DEBUG
  Serial.begin(115200);
  delay(200);
  Serial.println();
  Serial.println("Starting setup");
#endif

  pixels.begin();
  for (int i = 0; i < NEOPIXELS; i++)
      pixels.setPixelColor(i, pixels.Color(0, 0, 0));
  pixels.show();

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  wifi_connect();

  client.setServer(mqtt_server, 1883);
#ifdef NEOPIXEL_STATIC
  client.setCallback(callback);
#endif
  // warten bis die Verbindung zum MQTT-Server steht
  mqtt_connect();

  #ifdef DEBUG
    Serial.println();
    Serial.println("Starting loop");
    Serial.println();
  #endif
}


// --------------------------------------------------------
unsigned long next_job_starttime = millis();
unsigned long next_second = millis();
uint8_t cycle = 0;
uint8_t seconds = 0;
uint8_t led, i  = 0;

void loop() 
{
  // alle 5ms
  if (millis() >= next_job_starttime)
  {  
    // der nächste Job startet 5ms später
    next_job_starttime = millis() + 5;
    client.loop(); 
    cycle++;
  }

  // nach 25 * 5ms = 125ms
  if (cycle > 24)
  {
    cycle = 0;

    // nach jeweils 1 Sekunde
    // nur zum Testen mit Zufallswerten
    if (next_job_starttime > next_second)
    {
      seconds++;
      next_second += 1000;

      i  = (++i % 16);
      pixel[0] = (led % 16);
      pixel[1] = (random(16) * 4);
      pixel[2] = (random(16) * 4);
      pixel[3] = (random(16) * 4);
      pixel[5] = (random(4));
      set_mypixel(pixel);
      led++;
      led = led & 0x0F;
    }
    
  #ifdef NEOPIXEL_BLINK
    check_blink();
  #endif

    if (!(WiFi.status() == WL_CONNECTED))  wifi_connect();
    if (!client.connected()) mqtt_connect();
  }
}
